﻿using ID3_Sample_Code_Test.ID3global;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using CsvHelper;
using System.Globalization;

namespace ID3_Sample_Code_Test
{
    /*
         * This code is provided without warranty for the purpose of simplifying ID3global integration.  It is not intended for commercial use; you should write your own code for that purpose.
         * 
         * The service reference points to pilot, our sandbox environment.  If you need to test live, switch this reference.
         * 
         * You will need to ensure that you enter your authentication credentials above for these dummy methods to work.
         * 
         * If you want to test the XML sample requests, you will also need to ensure that you pass your credentials within those requests where indicated.  
         * Also note that you will need to ensure that you use your own profile GUID by replacing the one in the XML.
         * 
         * All code working at time of release: 25/02/2021
         * */

    class Program
    {
        /*
         * Bindings need to match the endpoint address, contract & names in App.config
         * Each binding has a different endpoint, so the name of the binding will need to be changed for each different method called
         * */
        static string wsAuthBinding = "wsHttpBinding_GlobalAuthenticate";
        static string basicAuthBinding = "basicHttpBinding_GlobalAuthenticate";
        static string basicCredBinding = "basicHttpBinding_GlobalCredentials";
        static string wsCredBinding = "wsHttpBinding_GlobalCredentials";
        static string basicSearchBinding = "basicHttpBinding_GlobalSearch";
        static string basicConfigBinding = "basicHttpBinding_GlobalConfig";

        static string username = @"";
        static string password = @"";    

        
        static void Main(string[] args)
        {
            //check that your credentials are working ok; make sure you've entered the credentials above!
            bool credentialsSuccess = TestCredentials();

            if(credentialsSuccess)
            {
                //now put some dummy data through and see the response
                TestAuthenticateSP();

                //use this to parse XML request & submit to AuthenticateSP
                TestAuthenticateSP_ReadXML(@"put full file path including extension here"); 

                //use this to parse the XML response from ID3global and output to console
                DeserialiseXml(@"put full file path including extension here");

                TestSanctionsAuthenticateSP();

                TestPEPAuthenticateSP();
            }

            Console.WriteLine("Testing Complete");
            Console.ReadKey();
        }

        static Boolean TestCredentials()
        {
            //Simple check that your credentials are working correctly using two different bindings

            Console.WriteLine("*\nCheckCredentials - Basic Binding");

            GlobalAccount checkCredentialsResult = ID3globalServiceCalls.CheckCredentials(basicCredBinding, username, password);

            if(checkCredentialsResult == null)
            {
                Console.WriteLine("Check Credentials failed; check username & password");

                return false;
            }
            else
            {
                Console.WriteLine("Credentials ok: " + checkCredentialsResult.OrgID.ToString());

                Console.WriteLine("\n\nCheckCredentials - WSHttpBinding\n");

                checkCredentialsResult = ID3globalServiceCalls.CheckCredentials(wsCredBinding, username, password);

                Console.WriteLine("Credentials ok: " + checkCredentialsResult.OrgID.ToString());

                return true;
            }
        }

        static void TestAuthenticateSP_ReadXML(string xmlFilePath)
        {
            // Serialises XML request for AuthenticateSP and submits to ID3global
             try
            {
                Console.WriteLine("*\nTestAuthenticateSP - Read XML");                

                var creds = XMLDeserialiser.ExtractUsernameAndPasswordFromXML(xmlFilePath);

                GlobalProfileIDVersion profileGuid = XMLDeserialiser.ExtractProfileGuidFromXML(xmlFilePath);

                //Reads XML file, converts to GlobalInputData, then submits to AuthenticateSP
                GlobalInputData globalInputData = XMLDeserialiser.ConvertXmlRequestToGlobalInputData(xmlFilePath);

                //now let's execute the calls

                Console.WriteLine("AuthenticateSP - Basic binding\n");

                ID3globalServiceCalls.AuthenticateSP(globalInputData, wsAuthBinding, profileGuid, creds.Item1, creds.Item2, "Customer 1 Basic binding via XML");
            }
            catch(Exception e)
            {
                Console.WriteLine(e.ToString());
            }
            

        }

        static void TestAuthenticateSP()
        {
            /*
             * Sample call to AuthenticateSP, requires no data to be set up.  
             * The data here is GBG test data for the Pilot test site and UK data set "Advanced Credit Header 155"; please replace with appropriate test data 
             * */
             
            Console.WriteLine("*\nTestAuthenticateSP");                                 

            // Here we construct the personal information of the subject we want to check
            GlobalPersonalDetails personalDetails = new GlobalPersonalDetails()
            {
                Forename = "Nander",
                Surname = "Casseter",
                DOBDay = 28,
                DOBMonth = 02,
                DOBYear = 1990
            };

            // Here we construct the address information of the subject we want to check
            GlobalAddress address = new GlobalAddress()
            {
                Building = "866",
                Street = "Downham Road",
                City ="Ely",
                Country = "United Kingdom",
                ZipPostcode = "CB6 1AF"
            };

            // Assemble that data together into the GlobalInputData class
            GlobalInputData globalInputData = new GlobalInputData()
            {
                Personal = new GlobalPersonal()
                {
                    PersonalDetails = personalDetails
                },
                Addresses = new GlobalAddresses()
                {
                    CurrentAddress = address,
                }
            };

            //this profile ID is from gb.co.uk on pilot; swap it out for your own

            GlobalProfileIDVersion profileGuid = new GlobalProfileIDVersion()
            {
                ID = new Guid("5e2e87d7-9381-4bfd-a8d6-3f24812be6c1"),
                Version = 0
            };

            //now let's execute the calls.  We will call the service twice, using Basic then WSHttp binding
            
            Console.WriteLine("AuthenticateSP - Basic binding\n");

            var results = ID3globalServiceCalls.AuthenticateSP(globalInputData, basicAuthBinding, profileGuid, username, password, "Customer 1 Basic");

            // Now we have our result data we can do what we want with it, you can return this as a class to your caller
            //  for now we are just going to pipe this out to the console.
            Console.WriteLine("Customer Ref: " + results.CustomerRef);
            Console.WriteLine("Band text: " + results.BandText);
            Console.WriteLine("Score: " + results.Score.ToString());
            Console.WriteLine("Authentication ID: " + results.AuthenticationID.ToString());
            Console.WriteLine("Timestamp: " + results.Timestamp.ToString());
        }

        static void TestSanctionsAuthenticateSP()
        {
            /*
             * Sample call to AuthenticateSP, requires no data to be set up.  
             * The data here is for a known Sanctions match, which will then trigger more detailed checks to be made
             * */

            Console.WriteLine("*\nTestAuthenticateSP with Sanctions");            

            // Here we construct the personal information of the subject we want to check
            GlobalPersonalDetails personalDetails = new GlobalPersonalDetails()
            {
                Forename = "Osama",
                Surname = "Bin Laden",
                DOBDay = 10,
                DOBMonth = 03,
                DOBYear = 1957
            };


            // Assemble that data together into the GlobalInputData class
            GlobalInputData globalInputData = new GlobalInputData()
            {
                Personal = new GlobalPersonal()
                {
                    PersonalDetails = personalDetails
                },
                Addresses = new GlobalAddresses() 
            };

            //this profile ID is from gb.co.uk on pilot; swap it out for your own

            GlobalProfileIDVersion profileGuid = new GlobalProfileIDVersion()
            {
                ID = new Guid("b56daf1f-91c2-4f9a-9572-b159cdd5dc8b"),
                Version = 0
            };


            //Assuming that a result contains a match for sanctions, all matching details will be shared

            GlobalResultData results = ID3globalServiceCalls.AuthenticateSP(globalInputData, wsAuthBinding, profileGuid, username, password, "Sanctions Check");

            Console.WriteLine("Customer Ref: " + results.CustomerRef);
            Console.WriteLine("Band text: " + results.BandText);
            Console.WriteLine("Score: " + results.Score.ToString());
            Console.WriteLine("Authentication ID: " + results.AuthenticationID.ToString());
            Console.WriteLine("Timestamp: " + results.Timestamp.ToString());

            //create a collection to store all results dynamically, ready for output
            List<OutputRecord> records = new List<OutputRecord>();

            if (results != null)
            {
                //In order to process a Sanctions check, we need to extract the SanctionsID values returned from GlobalResultData
                List<string> sanctionsIDList = new List<string>();

                try
                {
                    foreach (GlobalSanctionsResultCodes rc in results.ResultCodes)
                    {
                        if (rc != null)
                        {
                            foreach (GlobalSanctionsMatch m in rc.SanctionsMatches)
                            {
                                //only take the ID if it's digits only (ignore other results)
                                if (IsDigitsOnly(m.SanctionID)) sanctionsIDList.Add(m.SanctionID);
                            }
                        }

                    }
                }
                catch (Exception e)
                {
                    Console.WriteLine(e.ToString());
                }

                if (sanctionsIDList.Count > 0)
                {
                    //this is a hit on Sanction
                    //GetSanctionsIntelligence require the OrgID; this must be gathered from CheckCredentials
                    Guid OrgID = new Guid();
                    bool GotOrgID = false;

                    var r = ID3globalServiceCalls.CheckCredentials(wsCredBinding, username, password);

                    if (r != null)
                    {
                        OrgID = r.OrgID;
                        GotOrgID = true;
                    }

                    if (GotOrgID)
                    {
                        List<GlobalSanctionsEnforcementData> globalSanctionsEnforcementDatas = new List<GlobalSanctionsEnforcementData>();
                        foreach (string sanctionID in sanctionsIDList)
                        {
                            //get Sanctions Intelligence
                            GlobalSanctionsEnforcementData globalSanctionsEnforcementData = ID3globalServiceCalls.GetSanctionsIntelligence(basicSearchBinding, username, password, OrgID, sanctionID);

                            if (globalSanctionsEnforcementData != null)
                            {
                                globalSanctionsEnforcementDatas.Add(globalSanctionsEnforcementData);
                            }
                        }

                        //we now have a list of all the GlobalSanctionsEnforcementData generated by this AuthenticateSP call
                        foreach (GlobalSanctionsEnforcementData sanction in globalSanctionsEnforcementDatas)
                        {
                            try
                            {
                                Console.WriteLine("\n\n|Sanctions Match|\n");
                                Console.WriteLine(sanction.Fullname);

                                if (sanction.SanctionsBodies != null && sanction.SanctionsBodies.Length > 0)
                                {
                                    //need to make another call to fetch this info: GetSanctionsIssuer
                                    var globalSanctionsIssuers = ID3globalServiceCalls.GetSanctionsIssuer(basicConfigBinding, username, password);

                                    foreach (string sanctionBody in sanction.SanctionsBodies)
                                    {
                                        //Find the Sanction Issuer that matches
                                        var sanctionBodyName = globalSanctionsIssuers.Where(s => s.ID.ToString() == sanctionBody).FirstOrDefault().Name;

                                        Console.WriteLine("Sanction Body: " + sanctionBodyName);
                                        records.Add(new OutputRecord("Sanction Body", sanctionBodyName));
                                    }
                                }

                                foreach (string alias in sanction.Aliases)
                                {
                                    Console.WriteLine("Alias: " + alias);
                                    records.Add(new OutputRecord("Alias", alias));
                                }
                                foreach (GlobalSanctionsAddress address in sanction.SanctionsAddresses)
                                {
                                    Console.WriteLine(address.AddressLine);
                                    Console.WriteLine(address.City);
                                    Console.WriteLine(address.Country);

                                    records.Add(new OutputRecord("Address", address.AddressLine));
                                    records.Add(new OutputRecord("City", address.City));
                                    records.Add(new OutputRecord("Country", address.Country));
                                }

                                foreach (GlobalSanctionsDate date in sanction.SanctionsDates)
                                {
                                    Console.WriteLine(date.Day + "/" + date.Month + "/" + date.Year);
                                    records.Add(new OutputRecord("Sanction Date", date.Day + "/" + date.Month + "/" + date.Year));
                                }
                            }
                            catch (Exception e)
                            {
                                //missing some data
                            }
                        }
                    }
                }

            }
            if(records != null)
            {
                using (var writer = new StreamWriter(@"C:\ID3Handler\sanctionsoutput.csv"))
                using (var csv = new CsvWriter(writer, CultureInfo.InvariantCulture))
                {
                    csv.WriteRecords(records);
                }
            }            
        }

        static void TestPEPAuthenticateSP()
        {
            Console.WriteLine("*\nTestAuthenticateSP with PEP");
            //Sample call, requires no data to be set up 

            //Bindings need to match the endpoint address, contract & names in App.config                      

            // Here we construct the personal information of the subject we want to check
            GlobalPersonalDetails personalDetails = new GlobalPersonalDetails()
            {
                Forename = "Donald",
                Surname = "Trump",
                DOBDay = 14,
                DOBMonth = 06,
                DOBYear = 1946
            };

            // Assemble that data together into the GlobalInputData class
            GlobalInputData globalInputData = new GlobalInputData()
            {
                Personal = new GlobalPersonal()
                {
                    PersonalDetails = personalDetails
                },
                Addresses = new GlobalAddresses()
            };

            //this profile ID is from gb.co.uk on pilot; swap it out for your own

            GlobalProfileIDVersion profileGuid = new GlobalProfileIDVersion()
            {
                ID = new Guid("b56daf1f-91c2-4f9a-9572-b159cdd5dc8b"),
                Version = 0
            };

            //now let's execute the call

            GlobalResultData result = ID3globalServiceCalls.AuthenticateSP(globalInputData, basicAuthBinding, profileGuid, username, password, "Customer 1 Basic");

            //create a collection to store all results dynamically, ready for output
            List<OutputRecord> records = new List<OutputRecord>();

            if (result != null)
            {
                //In order to process a PEP check, we need to extract the PEPID values returned from GlobalResultData
                List<string> pepsIDList = new List<string>();

                try
                {
                    foreach (GlobalSanctionsResultCodes rc in result.ResultCodes)
                    {
                        if (rc != null)
                        {
                            //confusingly, ID3global uses "SanctionMatches" to refer to PEPs too
                            foreach (GlobalSanctionsMatch m in rc.SanctionsMatches)
                            {
                                //only take the ID if it's digits only (ignore other results)
                                if (IsDigitsOnly(m.SanctionID))
                                {
                                    pepsIDList.Add(m.SanctionID);
                                    Console.WriteLine("Sanction ID: " + m.SanctionID);
                                }
                            }
                        }
                    }
                }
                catch (Exception e)
                {
                    Console.WriteLine(e.ToString());
                }

                //GetPEPIntelligence / GetSanctionsIntelligence require the OrgID; this must be gathered from CheckCredentials

                Guid OrgID = new Guid();
                bool GotOrgID = false;

                if (pepsIDList.Count > 0)
                {
                    //this is a hit on PEPs
                    var r = ID3globalServiceCalls.CheckCredentials(wsCredBinding, username, password);

                    if (r != null)
                    {
                        OrgID = r.OrgID;
                        GotOrgID = true;
                    }

                    if (GotOrgID)
                    {
                        List<GlobalPEPIntelligenceData> pepDataList = new List<GlobalPEPIntelligenceData>();
                        foreach (string sanctionID in pepsIDList)
                        {
                            //get Sanctions Intelligence
                            GlobalPEPIntelligenceData pepData = ID3globalServiceCalls.GetPEPIntelligence(basicSearchBinding, username, password, OrgID, sanctionID);

                            if (pepData != null)
                            {
                                pepDataList.Add(pepData);
                            }
                        }

                        //we now have a list of all the GlobalSanctionsEnforcementData generated by this AuthenticateSP call
                        foreach (GlobalPEPIntelligenceData pep in pepDataList)
                        {
                            try
                            {
                                Console.WriteLine("\n\n|PEP Match|\n");
                                Console.WriteLine(pep.Fullname);
                                Console.WriteLine("Tier: " + pep.Tier);

                                records.Add(new OutputRecord("Full Name", pep.Fullname));
                                records.Add(new OutputRecord("Tier", pep.Tier.ToString()));

                                foreach (string alias in pep.Aliases)
                                {
                                    Console.WriteLine("Alias: " + alias);

                                    records.Add(new OutputRecord("Alias", alias));
                                }
                                foreach (GlobalSanctionsAddress address in pep.SanctionsAddresses)
                                {
                                    Console.WriteLine(address.AddressLine);
                                    Console.WriteLine(address.City);
                                    Console.WriteLine(address.Country);

                                    records.Add(new OutputRecord("Address", address.AddressLine));
                                    records.Add(new OutputRecord("City", address.City));
                                    records.Add(new OutputRecord("Country", address.Country));
                                }

                                foreach (var position in pep.SanctionsPositions)
                                {
                                    Console.WriteLine(position.Position + ", " + position.Country);

                                    records.Add(new OutputRecord(position.Position, position.Country));
                                }

                                foreach (GlobalSanctionsDate date in pep.SanctionsDates)
                                {
                                    Console.WriteLine(date.Day + "/" + date.Month + "/" + date.Year);

                                    records.Add(new OutputRecord("Sanction Date", date.Day + "/" + date.Month + "/" + date.Year));
                                }
                            }
                            catch (Exception e)
                            {
                                //missing some data
                            }
                        }
                    }
                }                
            }
                

            if (records != null)
            {
                using (var writer = new StreamWriter(@"C:\ID3Handler\pepsoutput.csv"))
                using (var csv = new CsvWriter(writer, CultureInfo.InvariantCulture))
                {
                    csv.WriteRecords(records);
                }
            }               
        }

        static void DeserialiseXml(string filepath)
        {
            //reads XML response from ID3global AuthenticateSP and outputs values to console

            Console.WriteLine("*\nDeserialise XML");
            try
            {
                GlobalResultData results = XMLDeserialiser.ConvertXmlResponseToGlobalResultData(filepath);

                if(results == null)
                {
                    Console.WriteLine("Something went wrong deserialising the XML");
                }
                else
                {
                    Console.WriteLine("Profile: " + results.ProfileName);
                    Console.WriteLine("Customer Ref: " + results.CustomerRef);
                    Console.WriteLine("Band text: " + results.BandText);
                    Console.WriteLine("Score: " + results.Score.ToString());
                    Console.WriteLine("Authentication ID: " + results.AuthenticationID.ToString());
                    Console.WriteLine("Timestamp: " + results.Timestamp.ToString());
                }
               
            }
           catch(Exception e)
            {
                Console.WriteLine(e.ToString());
            }


        }

        static bool IsDigitsOnly(string str)
        {
            foreach (char c in str)
            {
                if (c < '0' || c > '9')
                    return false;
            }

            return true;
        }
    }
}
